#!/usr/bin/env python
#-*- coding: utf-8 -*-

from distutils.core import setup

setup(
    name='FrameDuino',
    version='0.1',
    description='Pinguino Board Framework',
    author='Adán Mauri Ungaro',
    author_email='adan.mauri@gmail.com',
    url='https://github.com/adanmauri/frameduino',
    packages=['frameduino', 'frameduino.drivers'],
    license='GPLv3'    
)
